@@ -17,13 +17,13 @@
 # pylint: enable=line-too-long
 # pylint: disable=g-import-not-at-top,g-bad-import-order,wrong-import-position
 
-__version__ = "0.0.0.1"
+__version__ = "0.0.1"
 
 from .assets import *
 from .assets.utils import *
 from .config import Config
-from .engine import BlenderEngine, GodotEngine, PyVistaEngine, UnityEngine
-from .rl import ParallelRLEnv, RLEnv
+from .engine import *
+from .rl import MultiProcessRLEnv, ParallelRLEnv, RLEnv
 from .scene import Scene
 from .utils import logging
 
