@@ -17,7 +17,7 @@
 import io
 import os
 import tempfile
-from typing import ByteString, List, Optional
+from typing import ByteString, List, Optional, Union
 
 import numpy as np
 import PIL.Image
@@ -143,7 +143,7 @@ def get_texture_as_pillow(gltf_scene: GLTF, texture_info: Optional[TextureInfo])
             resource.load()
         data = resource.data
 
-    image = PIL.Image.open(io.BytesIO(data))  # TODO checkk all this image stuff
+    image = PIL.Image.open(io.BytesIO(data))  # TODO check all this image stuff
 
     return image
 
@@ -166,7 +166,7 @@ def get_texture_as_pyvista(gltf_scene: GLTF, texture_info: Optional[TextureInfo]
             filename = gltf_image.mimeType.replace("/", ".")  # Will convert mimeType 'image/png' in 'image.png'
             filepath = os.path.join(tmpdirname, filename)
             with open(filepath, "wb") as file:
-                file.write(buffer_bytes)
+                file.write(bytearray(buffer_bytes))
             texture = pv.read_texture(filepath)
     else:
         resource = gltf_scene.get_resource(gltf_image.uri)
@@ -181,7 +181,10 @@ def get_texture_as_pyvista(gltf_scene: GLTF, texture_info: Optional[TextureInfo]
 
 # Build a tree of simulate nodes from a GLTF object
 def build_node_tree(
-    gltf_scene: GLTF, pyvista_meshes: pv.MultiBlock, gltf_node_id: int, parent: Optional[Asset] = None
+    gltf_scene: GLTF,
+    pyvista_meshes: Union[np.ndarray, pv.MultiBlock],
+    gltf_node_id: int,
+    parent: Optional["Asset"] = None,
 ) -> List:
     """Build the node tree of simulate objects from the GLTF scene"""
     gltf_model = gltf_scene.model
@@ -285,7 +288,6 @@ def build_node_tree(
         # Let's add a mesh
         gltf_mesh = gltf_model.meshes[gltf_node.mesh]
         primitives = gltf_mesh.primitives
-        base_name = common_kwargs["name"]
         common_kwargs["with_rigid_body"] = False
         common_kwargs["with_articulation_body"] = False
         if len(primitives) > 1:
@@ -294,31 +296,36 @@ def build_node_tree(
             mesh = pyvista_meshes[f"Mesh_{gltf_node.mesh}"][0]  # A pv.PolyData
 
         common_kwargs["mesh"] = mesh
-        if len(primitives) > 1:
-            name = f"{base_name}_{mat.name}"
-            common_kwargs["name"] = name
 
         if gltf_collider is None:
-            material_id = primitives[0].material
-            if material_id is not None:
-                mat = gltf_model.materials[material_id]
-                pbr = mat.pbrMetallicRoughness
+            material_ids = [p.material for p in primitives]
+            scene_material = []
+            for material_id in material_ids:
+                if material_id is not None:
+                    mat = gltf_model.materials[material_id]
+                    pbr = mat.pbrMetallicRoughness
 
-                scene_material = Material(
-                    name=mat.name,
-                    base_color=pbr.baseColorFactor,
-                    base_color_texture=get_texture_as_pyvista(gltf_scene, pbr.baseColorTexture),
-                    metallic_factor=pbr.metallicFactor,
-                    metallic_roughness_texture=get_texture_as_pyvista(gltf_scene, pbr.metallicRoughnessTexture),
-                    normal_texture=get_texture_as_pyvista(gltf_scene, mat.normalTexture),
-                    occlusion_texture=get_texture_as_pyvista(gltf_scene, mat.occlusionTexture),
-                    emissive_factor=mat.emissiveFactor,
-                    emissive_texture=get_texture_as_pyvista(gltf_scene, mat.emissiveTexture),
-                    alpha_mode=mat.alphaMode,
-                    alpha_cutoff=mat.alphaCutoff,
-                )
-            else:
-                scene_material = Material()
+                    scene_material.append(
+                        Material(
+                            name=mat.name,
+                            base_color=pbr.baseColorFactor,
+                            base_color_texture=get_texture_as_pyvista(gltf_scene, pbr.baseColorTexture),
+                            metallic_factor=pbr.metallicFactor,
+                            metallic_roughness_texture=get_texture_as_pyvista(
+                                gltf_scene, pbr.metallicRoughnessTexture
+                            ),
+                            normal_texture=get_texture_as_pyvista(gltf_scene, mat.normalTexture),
+                            occlusion_texture=get_texture_as_pyvista(gltf_scene, mat.occlusionTexture),
+                            emissive_factor=mat.emissiveFactor,
+                            emissive_texture=get_texture_as_pyvista(gltf_scene, mat.emissiveTexture),
+                            alpha_mode=mat.alphaMode,
+                            alpha_cutoff=mat.alphaCutoff,
+                        )
+                    )
+                else:
+                    scene_material.append(Material())
+            if len(scene_material) == 1:
+                scene_material = scene_material[0]
             common_kwargs["material"] = scene_material
 
         if gltf_collider is not None:
@@ -374,7 +381,7 @@ def load_gltf_as_tree(
     repo_id: Optional[str] = None,
     subfolder: Optional[str] = None,
     revision: Optional[str] = None,
-) -> List[Asset]:
+) -> Union[List[List], List["Asset"]]:
     """
     Loading function to create a tree of asset nodes from a GLTF file.
     Return a list of the main nodes in the GLTF files (often only one main node).
