@@ -15,7 +15,7 @@
 # Lint as: python3
 """ A simulate Scene Object."""
 import itertools
-from typing import List, Optional, Union
+from typing import Any, List, Optional, Tuple, Union
 
 import numpy as np
 import pyvista as pv
@@ -26,14 +26,18 @@ from .asset import Asset
 from .collider import Collider
 from .material import Material
 from .procgen.prims import generate_prims_maze
-from .procgen.wfc import generate_2d_map, generate_map
 from .rigid_body import RigidBodyComponent
 
 
 logger = logging.get_logger(__name__)
 
 
-def translate(surf, center=(0.0, 0.0, 0.0), new_direction=(1.0, 0.0, 0.0), original_direction=(1.0, 0.0, 0.0)):
+def translate(
+    surf,
+    center: Union[Tuple[float, float, float], List[float], np.ndarray] = (0.0, 0.0, 0.0),
+    new_direction: Union[Tuple[float, float, float], List[float], np.ndarray] = (1.0, 0.0, 0.0),
+    original_direction: Union[Tuple[float, float, float], List[float], np.ndarray] = (1.0, 0.0, 0.0),
+):
     """Translate and orient a mesh to a new center and direction.
 
     By default, the input mesh is considered centered at the origin
@@ -48,10 +52,10 @@ def translate(surf, center=(0.0, 0.0, 0.0), new_direction=(1.0, 0.0, 0.0), origi
     if s > 0:
         c = np.dot(original_direction, new_direction)
         vx = np.array([[0, -v[2], v[1]], [v[2], 0, -v[0]], [-v[1], v[0], 0]])
-        R = np.eye(3) + vx + vx.dot(vx) * ((1 - c) / (s**2))
+        r = np.eye(3) + vx + vx.dot(vx) * ((1 - c) / (s**2))
 
         trans = np.zeros((4, 4))
-        trans[:3, :3] = R
+        trans[:3, :3] = r
         trans[3, 3] = 1
         surf.transform(trans)
 
@@ -77,18 +81,18 @@ class Object3D(Asset):
 
     def __init__(
         self,
-        mesh: Optional[Union[pv.UnstructuredGrid, pv.MultiBlock]] = None,
-        material: Optional[Material] = None,
+        mesh: Optional[Union[pv.UnstructuredGrid, pv.MultiBlock, pv.PolyData, pv.DataSet]] = None,
+        material: Optional[Union[Material, List[Material]]] = None,
         name: Optional[str] = None,
         position: Optional[List[float]] = None,
-        is_actor: Optional[bool] = False,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
+        is_actor: bool = False,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
         set_mesh_direction: Optional[List[float]] = None,
         original_mesh_direction: Optional[List[float]] = None,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
         super().__init__(name=name, position=position, is_actor=is_actor, parent=parent, children=children, **kwargs)
 
@@ -107,13 +111,21 @@ class Object3D(Asset):
             translate(mesh, (0, 0, 0), new_direction=set_mesh_direction, original_direction=original_mesh_direction)
 
         # Avoid having averaging normals at shared points
-        # (default pyvista behavior:https://docs.pyvista.org/api/core/_autosummary/pyvista.PolyData.compute_normals.html)
+        # (pyvista behavior:https://docs.pyvista.org/api/core/_autosummary/pyvista.PolyData.compute_normals.html)
         if self.mesh is not None:
-            self.mesh.compute_normals(inplace=True, cell_normals=False, split_vertices=True)
+            if isinstance(self.mesh, pv.MultiBlock):
+                for i in range(self.mesh.n_blocks):
+                    self.mesh[i].compute_normals(inplace=True, cell_normals=False, split_vertices=True)
+            else:
+                self.mesh.compute_normals(inplace=True, cell_normals=False, split_vertices=True)
 
         self.material = material if material is not None else Material()
 
-    def copy(self, with_children=True, **kwargs):
+        if isinstance(self.material, (list, tuple)) or isinstance(self.mesh, pv.MultiBlock):
+            if not isinstance(self.mesh, pv.MultiBlock) or len(self.material) != self.mesh.n_blocks:
+                raise ValueError("Number of materials must match number of blocks in mesh")
+
+    def copy(self, with_children: bool = True, **kwargs: Any) -> "Object3D":
         """Copy an Object3D node in a new (returned) object.
 
         By default, mesh and materials are copied in respectively new mesh and material.
@@ -157,18 +169,17 @@ class Object3D(Asset):
 
         return instance_copy
 
-    def _post_name_change(self, value):
+    def _post_name_change(self, value: Any):
         """NodeMixing method call after changing the name of a node."""
         for node in self.tree_children:
             if isinstance(node, Collider):
                 node.name = self.name + "_collider"  # Let's keep the name of the collider in sync
 
-        self.tree_root._check_all_names_unique()  # Check that all names are unique in the tree
-
     def _repr_info_str(self) -> str:
         """Used to add additional information to the __repr__ method."""
-        mesh_str = ""
-        if hasattr(self, "mesh") and self.mesh is not None:
+        if isinstance(self.mesh, pv.MultiBlock):
+            mesh_str = f"Mesh(Multiblock, n_blocks={self.mesh.n_blocks}"
+        else:
             mesh_str = f"Mesh(points={self.mesh.n_points}, cells={self.mesh.n_cells})"
         material_str = ""
         if hasattr(self, "material") and self.material is not None:
@@ -217,23 +228,23 @@ class Plane(Object3D):
 
     def __init__(
         self,
-        i_size: Optional[float] = 10,
-        j_size: Optional[float] = 10,
-        i_resolution: Optional[int] = 1,
-        j_resolution: Optional[int] = 1,
+        i_size: float = 10,
+        j_size: float = 10,
+        i_resolution: int = 1,
+        j_resolution: int = 1,
         name: Optional[str] = None,
         position: Optional[List[float]] = None,
-        is_actor: Optional[bool] = False,
+        is_actor: bool = False,
         with_collider: bool = True,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
         set_mesh_direction: Optional[List[float]] = None,
         collider_thickness: Optional[float] = None,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
-        original_mesh_direction = (0, -1, 0)
+        original_mesh_direction = [0, -1, 0]
         mesh = pv.Plane(
             direction=original_mesh_direction,
             i_size=i_size,
@@ -261,8 +272,8 @@ class Plane(Object3D):
             collider = Collider(
                 name=self.name + "_collider",
                 type="box",
-                bounding_box=(i_size, collider_thickness, j_size),
-                position=(0, -collider_thickness / 2, 0),
+                bounding_box=[i_size, collider_thickness, j_size],
+                position=[0, -collider_thickness / 2, 0],
             )
             self.tree_children = (children if children is not None else []) + [collider]
 
@@ -315,23 +326,23 @@ class Sphere(Object3D):
     def __init__(
         self,
         position: Optional[List[float]] = None,
-        radius: Optional[float] = 1.0,
-        theta_resolution: Optional[int] = 10,
-        phi_resolution: Optional[int] = 10,
-        start_theta: Optional[float] = 0,
-        end_theta: Optional[float] = 360,
-        start_phi: Optional[float] = 0,
-        end_phi: Optional[float] = 180,
-        sphere_type: Optional[str] = "uv",
+        radius: float = 1.0,
+        theta_resolution: int = 10,
+        phi_resolution: int = 10,
+        start_theta: float = 0.0,
+        end_theta: float = 360.0,
+        start_phi: float = 0.0,
+        end_phi: float = 180.0,
+        sphere_type: str = "uv",
         with_collider: bool = True,
         name: Optional[str] = None,
-        is_actor: Optional[bool] = False,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
+        is_actor: bool = False,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
         set_mesh_direction: Optional[List[float]] = None,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
         if sphere_type not in ["uv", "ico"]:
             raise ValueError("Sphere type should be one of 'uv' or 'ico'.")
@@ -357,7 +368,7 @@ class Sphere(Object3D):
             position=position,
             is_actor=is_actor,
             set_mesh_direction=set_mesh_direction,
-            original_mesh_direction=(0, 1, 0),
+            original_mesh_direction=[0, 1, 0],
             with_rigid_body=with_rigid_body,
             with_articulation_body=with_articulation_body,
             parent=parent,
@@ -369,7 +380,7 @@ class Sphere(Object3D):
             collider = Collider(
                 name=self.name + "_collider",
                 type="sphere",
-                bounding_box=(radius, radius, radius),
+                bounding_box=[radius, radius, radius],
             )
             self.tree_children = (children if children is not None else []) + [collider]
 
@@ -416,20 +427,20 @@ class Capsule(Object3D):
     def __init__(
         self,
         position: Optional[List[float]] = None,
-        height: Optional[float] = 1.0,
-        radius: Optional[float] = 0.2,
-        theta_resolution: Optional[int] = 4,
-        phi_resolution: Optional[int] = 4,
-        sphere_type: Optional[str] = "uv",
+        height: float = 1.0,
+        radius: float = 0.2,
+        theta_resolution: int = 4,
+        phi_resolution: int = 4,
+        sphere_type: str = "uv",
         with_collider: bool = True,
         name: Optional[str] = None,
-        is_actor: Optional[bool] = False,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
+        is_actor: bool = False,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
         set_mesh_direction: Optional[List[float]] = None,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
         if sphere_type not in ["uv", "ico"]:
             raise ValueError("Sphere type should be one of 'uv' or 'ico'.")
@@ -438,7 +449,7 @@ class Capsule(Object3D):
 
         capsule = vtkCapsuleSource()  # TODO pyvista capsules are aranged on the side
         capsule.SetRadius(radius)
-        capsule.SetCylinderLength(max(0, height - radius * 2))
+        capsule.SetCylinderLength(max(0.0, height - radius * 2))
         capsule.SetThetaResolution(theta_resolution)
         capsule.SetPhiResolution(phi_resolution)
         capsule.SetLatLongTessellation(bool(sphere_type == "uv"))
@@ -452,7 +463,7 @@ class Capsule(Object3D):
             position=position,
             is_actor=is_actor,
             set_mesh_direction=set_mesh_direction,
-            original_mesh_direction=(0, 1, 0),
+            original_mesh_direction=[0, 1, 0],
             with_rigid_body=with_rigid_body,
             with_articulation_body=with_articulation_body,
             parent=parent,
@@ -464,7 +475,7 @@ class Capsule(Object3D):
             collider = Collider(
                 name=self.name + "_collider",
                 type="capsule",
-                bounding_box=(radius, height, radius),
+                bounding_box=[radius, height, radius],
             )
             self.tree_children = (children if children is not None else []) + [collider]
 
@@ -505,21 +516,21 @@ class Cylinder(Object3D):
 
     def __init__(
         self,
-        height: Optional[float] = 1.0,
-        radius: Optional[float] = 1.0,
-        resolution: Optional[int] = 16,
-        capping: Optional[bool] = True,
+        height: float = 1.0,
+        radius: float = 1.0,
+        resolution: int = 16,
+        capping: bool = True,
         name: Optional[str] = None,
         position: Optional[List[float]] = None,
         set_mesh_direction: Optional[List[float]] = None,
-        is_actor: Optional[bool] = False,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        is_actor: bool = False,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
-        original_mesh_direction = (0, 1, 0)
+        original_mesh_direction = [0, 1, 0]
         mesh = pv.Cylinder(
             direction=original_mesh_direction, radius=radius, height=height, resolution=resolution, capping=capping
         )
@@ -560,7 +571,7 @@ class Box(Object3D):
             => bounds are ``(-xSize/2, xSize/2, ySize/2, ySize/2, -zSize/2, zSize/2)``
         - a single float: size
             => bounds are ``(-size/2, size/2, size/2, size/2, -size/2, size/2)``
-        If no value is provide, create a centered unit box
+        If no value is provided, create a centered unit box
 
     level : int, optional
         Level of subdivision of the faces.
@@ -582,18 +593,18 @@ class Box(Object3D):
     def __init__(
         self,
         bounds: Optional[Union[float, List[float]]] = None,
-        level: Optional[int] = 0,
-        quads: Optional[bool] = True,
+        level: int = 0,
+        quads: bool = True,
         with_collider: bool = True,
         name: Optional[str] = None,
         position: Optional[List[float]] = None,
         set_mesh_direction: Optional[List[float]] = None,
-        is_actor: Optional[bool] = False,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        is_actor: bool = False,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
         if bounds is None:
             bounds = (-0.5, 0.5, -0.5, 0.5, -0.5, 0.5)
@@ -617,7 +628,7 @@ class Box(Object3D):
             position=position,
             is_actor=is_actor,
             set_mesh_direction=set_mesh_direction,
-            original_mesh_direction=(0, 1, 0),
+            original_mesh_direction=[0, 1, 0],
             with_rigid_body=with_rigid_body,
             with_articulation_body=with_articulation_body,
             parent=parent,
@@ -626,12 +637,12 @@ class Box(Object3D):
         )
 
         if with_collider:
-            bounding_box = (bounds[1] - bounds[0], bounds[3] - bounds[2], bounds[5] - bounds[4])
-            offset = (
+            bounding_box = [bounds[1] - bounds[0], bounds[3] - bounds[2], bounds[5] - bounds[4]]
+            offset = [
                 (bounds[0] + bounds[1]) / 2.0,
                 (bounds[2] + bounds[3]) / 2.0,
                 (bounds[4] + bounds[5]) / 2.0,
-            )
+            ]
             collider = Collider(name=self.name + "_collider", type="box", bounding_box=bounding_box, offset=offset)
             self.tree_children = (children if children is not None else []) + [collider]
 
@@ -670,20 +681,20 @@ class Cone(Object3D):
 
     def __init__(
         self,
-        height: Optional[float] = 1.0,
-        radius: Optional[float] = 1.0,
-        resolution: Optional[int] = 6,
+        height: float = 1.0,
+        radius: float = 1.0,
+        resolution: int = 6,
         name: Optional[str] = None,
         position: Optional[List[float]] = None,
         set_mesh_direction: Optional[List[float]] = None,
-        is_actor: Optional[bool] = False,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        is_actor: bool = False,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
-        original_mesh_direction = (0, 1, 0)
+        original_mesh_direction = [0, 1, 0]
         mesh = pv.Cone(direction=original_mesh_direction, height=height, radius=radius, resolution=resolution)
         super().__init__(
             mesh=mesh,
@@ -728,15 +739,15 @@ class Line(Object3D):
         self,
         pointa: Optional[List[float]] = None,
         pointb: Optional[List[float]] = None,
-        resolution: Optional[int] = 1,
+        resolution: int = 1,
         name: Optional[str] = None,
-        is_actor: Optional[bool] = False,
+        is_actor: bool = False,
         set_mesh_direction: Optional[List[float]] = None,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
         if pointa is None:
             pointa = [-1.0, 0.0, 0.0]
@@ -749,7 +760,7 @@ class Line(Object3D):
             name=name,
             is_actor=is_actor,
             set_mesh_direction=set_mesh_direction,
-            original_mesh_direction=(1, 0, 0),
+            original_mesh_direction=[1, 0, 0],
             with_rigid_body=with_rigid_body,
             with_articulation_body=with_articulation_body,
             parent=parent,
@@ -780,13 +791,13 @@ class MultipleLines(Object3D):
         self,
         points: Optional[List[List[float]]] = None,
         name: Optional[str] = None,
-        is_actor: Optional[bool] = False,
+        is_actor: bool = False,
         set_mesh_direction: Optional[List[float]] = None,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
         if points is None:
             points = [[-1.0, 0.0, 0.0], [1.0, 0.0, 0.0]]
@@ -797,7 +808,7 @@ class MultipleLines(Object3D):
             name=name,
             is_actor=is_actor,
             set_mesh_direction=set_mesh_direction,
-            original_mesh_direction=(1, 0, 0),
+            original_mesh_direction=[1, 0, 0],
             with_rigid_body=with_rigid_body,
             with_articulation_body=with_articulation_body,
             parent=parent,
@@ -840,17 +851,17 @@ class Tube(Object3D):
         self,
         pointa: Optional[List[float]] = None,
         pointb: Optional[List[float]] = None,
-        resolution: Optional[int] = 1,
-        radius: Optional[float] = 1.0,
-        n_sides: Optional[int] = 16,
+        resolution: int = 1,
+        radius: float = 1.0,
+        n_sides: int = 16,
         name: Optional[str] = None,
-        is_actor: Optional[bool] = False,
+        is_actor: bool = False,
         set_mesh_direction: Optional[List[float]] = None,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
         if pointa is None:
             pointa = [-1.0, 0.0, 0.0]
@@ -863,7 +874,7 @@ class Tube(Object3D):
             name=name,
             is_actor=is_actor,
             set_mesh_direction=set_mesh_direction,
-            original_mesh_direction=(0, 1, 0),
+            original_mesh_direction=[0, 1, 0],
             with_rigid_body=with_rigid_body,
             with_articulation_body=with_articulation_body,
             parent=parent,
@@ -908,14 +919,14 @@ class Polygon(Object3D):
         points: List[List[float]],
         position: Optional[List[float]] = None,
         name: Optional[str] = None,
-        is_actor: Optional[bool] = False,
+        is_actor: bool = False,
         set_mesh_direction: Optional[List[float]] = None,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
         with_collider: bool = False,
-        **kwargs,
+        **kwargs: Any,
     ):
         from vtkmodules.vtkCommonDataModel import vtkCellArray, vtkPolyData, vtkPolygon
 
@@ -934,11 +945,11 @@ class Polygon(Object3D):
         polygons.InsertNextCell(polygon)
 
         # Create a PolyData
-        polygonPolyData = vtkPolyData()
-        polygonPolyData.SetPoints(v_points)
-        polygonPolyData.SetPolys(polygons)
+        polygon_poly_data = vtkPolyData()
+        polygon_poly_data.SetPoints(v_points)
+        polygon_poly_data.SetPolys(polygons)
 
-        mesh = pv.PolyData(polygonPolyData)
+        mesh = pv.PolyData(polygon_poly_data)
 
         super().__init__(
             mesh=mesh,
@@ -946,7 +957,7 @@ class Polygon(Object3D):
             position=position,
             is_actor=is_actor,
             set_mesh_direction=set_mesh_direction,
-            original_mesh_direction=(0, 1, 0),
+            original_mesh_direction=[0, 1, 0],
             with_rigid_body=with_rigid_body,
             with_articulation_body=with_articulation_body,
             parent=parent,
@@ -994,19 +1005,19 @@ class RegularPolygon(Object3D):
 
     def __init__(
         self,
-        radius: Optional[float] = 1.0,
-        n_sides: Optional[int] = 6,
+        radius: float = 1.0,
+        n_sides: int = 6,
         position: Optional[List[float]] = None,
         name: Optional[str] = None,
-        is_actor: Optional[bool] = False,
+        is_actor: bool = False,
         set_mesh_direction: Optional[List[float]] = None,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
-        original_mesh_direction = (0, 1, 0)
+        original_mesh_direction = [0, 1, 0]
         mesh = pv.Polygon(radius=radius, normal=original_mesh_direction, n_sides=n_sides)
 
         super().__init__(
@@ -1066,21 +1077,21 @@ class Ring(Object3D):
     # TODO(thomas) add back center and normal and see how to handle that for 2D/3D stuff
     def __init__(
         self,
-        inner: Optional[float] = 0.25,
-        outer: Optional[float] = 0.5,
-        r_res: Optional[int] = 1,
-        c_res: Optional[int] = 6,
+        inner: float = 0.25,
+        outer: float = 0.5,
+        r_res: int = 1,
+        c_res: int = 6,
         name: Optional[str] = None,
         position: Optional[List[float]] = None,
         set_mesh_direction: Optional[List[float]] = None,
-        is_actor: Optional[bool] = False,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        is_actor: bool = False,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
-        original_mesh_direction = (0, 1, 0)
+        original_mesh_direction = [0, 1, 0]
         mesh = pv.Disc(inner=inner, outer=outer, normal=original_mesh_direction, r_res=r_res, c_res=c_res)
 
         super().__init__(
@@ -1129,21 +1140,21 @@ class Text3D(Object3D):
 
     def __init__(
         self,
-        string: Optional[str] = "Hello",
-        depth: Optional[float] = 0.5,
+        string: str = "Hello",
+        depth: float = 0.5,
         name: Optional[str] = None,
         position: Optional[List[float]] = None,
         set_mesh_direction: Optional[List[float]] = None,
-        is_actor: Optional[bool] = False,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        is_actor: bool = False,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
         mesh = pv.Text3D(string=string, depth=depth)
         mesh.rotate_y(-90, inplace=True)
-        original_mesh_direction = (0, 0, -1)
+        original_mesh_direction = [0, 0, -1]
         translate(mesh, (0, 0, 0), new_direction=original_mesh_direction)
 
         super().__init__(
@@ -1184,13 +1195,13 @@ class Triangle(Object3D):
         self,
         points: Optional[List[List[float]]] = None,
         name: Optional[str] = None,
-        is_actor: Optional[bool] = False,
+        is_actor: bool = False,
         set_mesh_direction: Optional[List[float]] = None,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
         mesh = pv.Triangle(points=points)
 
@@ -1199,7 +1210,7 @@ class Triangle(Object3D):
             name=name,
             is_actor=is_actor,
             set_mesh_direction=set_mesh_direction,
-            original_mesh_direction=(0, 1, 0),
+            original_mesh_direction=[0, 1, 0],
             with_rigid_body=with_rigid_body,
             with_articulation_body=with_articulation_body,
             parent=parent,
@@ -1230,13 +1241,13 @@ class Rectangle(Object3D):
         self,
         points: Optional[List[List[float]]] = None,
         name: Optional[str] = None,
-        is_actor: Optional[bool] = False,
+        is_actor: bool = False,
         set_mesh_direction: Optional[List[float]] = None,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
         mesh = pv.Rectangle(points=points)
 
@@ -1245,7 +1256,7 @@ class Rectangle(Object3D):
             name=name,
             is_actor=is_actor,
             set_mesh_direction=set_mesh_direction,
-            original_mesh_direction=(0, 1, 0),
+            original_mesh_direction=[0, 1, 0],
             with_rigid_body=with_rigid_body,
             with_articulation_body=with_articulation_body,
             parent=parent,
@@ -1285,21 +1296,21 @@ class Circle(Object3D):
 
     def __init__(
         self,
-        radius: Optional[float] = 0.5,
-        resolution: Optional[int] = 100,
+        radius: float = 0.5,
+        resolution: int = 100,
         name: Optional[str] = None,
         position: Optional[List[float]] = None,
-        is_actor: Optional[bool] = False,
+        is_actor: bool = False,
         set_mesh_direction: Optional[List[float]] = None,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
         mesh = pv.Circle(radius=radius, resolution=resolution)
         mesh.rotate_y(-90, inplace=True)
-        original_mesh_direction = (0, 1, 0)
+        original_mesh_direction = [0, 1, 0]
         translate(mesh, (0, 0, 0), new_direction=original_mesh_direction)
 
         super().__init__(
@@ -1357,12 +1368,12 @@ class StructuredGrid(Object3D):
         name: Optional[str] = None,
         position: Optional[List[float]] = None,
         set_mesh_direction: Optional[List[float]] = None,
-        is_actor: Optional[bool] = False,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
+        is_actor: bool = False,
+        with_rigid_body: bool = False,
+        with_articulation_body: bool = False,
+        parent: Optional["Asset"] = None,
+        children: Optional[Union["Asset", List["Asset"]]] = None,
+        **kwargs: Any,
     ):
         if not isinstance(x, np.ndarray):
             x = np.array(x)
@@ -1373,7 +1384,7 @@ class StructuredGrid(Object3D):
 
         # If it is a structured grid, extract the surface mesh (PolyData)
         mesh = pv.StructuredGrid(x, y, z).extract_surface()
-        original_mesh_direction = (0, 1, 0)
+        original_mesh_direction = [0, 1, 0]
         translate(mesh, (0, 0, 0), new_direction=original_mesh_direction, original_direction=(0, 1, 0))
 
         super().__init__(
@@ -1391,199 +1402,23 @@ class StructuredGrid(Object3D):
         )
 
 
-class ProcgenGrid(Object3D):
-    """Create a procedural generated 3D grid (structured plane) from
-        tiles / previous map.
-
-    Parameters
-    ----------
-    sample_map : np.ndarray or python list of lists of floats
-        Map to procedurally generate from.
-
-    specific_map: np.ndarray or python list of lists of floats
-        Map to show as it is.
-
-    tiles : list of tiles
-        Tiles for procedural generation when using generation from tiles and neighbors definitions.
-        Tiles must be NxN np.ndarray that define height maps. In the future, we plan to support
-        more generic tiles.
-
-    neighbors: list of available neighbors for each tile
-        Expects pair of tiles.
-
-    symmetries: list of char
-        Expects a list of symmetry definitions. If passed, you must define a symmetry for each tile.
-        Possible symmetries are "X", "I" / "/", "T" / "L", another character, and each character defines
-        the tile with the same level of symmetry as the character:
-        - X: has symmetry in all ways. So it has 1 different format.
-        - I / `/`: when rotated, it's different from the original tile. Has 2 different formats.
-        - T / L: Has 4 different formats.
-        - other characters: the algorithm supposes it has 8 different formats.
-
-    weights: list of floats
-        sampling weights for each of the tiles
-
-    width: int
-        width of the generated map
-
-    height: int
-        height of the generated map
-
-    shallow: bool
-        Indicates whether procedural generation mesh should be generated in simenv or not.
-        When it's true, we just return the map returned by the algorithm without
-        actually creating the mesh in simenv.
-        Created for the purpose of optimizing certain environments such as XLand.
-
-    seed: int
-        Random seed used for procedural generation.
-
-    algorithm_args: dict
-        Extra arguments to be passed to the procedural generation.
-
-    Returns
-    -------
-
-    Examples
-    --------
-
-    """
-
+class ProcGenPrimsMaze3D(Asset):
     __NEW_ID = itertools.count()  # Singleton to count instances of the classes for automatic naming
 
     def __init__(
         self,
-        sample_map: Union[np.ndarray, List[List[List[int]]]] = None,
-        specific_map: Union[np.ndarray, List[List[List[int]]]] = None,
-        tiles: Optional[List] = None,
-        neighbors: Optional[List] = None,
-        symmetries: Optional[List] = None,
-        weights: Optional[List] = None,
-        width: Optional[int] = 9,
-        height: Optional[int] = 9,
-        shallow: Optional[bool] = False,
-        algorithm_args: Optional[dict] = None,
-        seed: int = None,
+        width: int,
+        depth: int,
         name: Optional[str] = None,
-        is_actor: Optional[bool] = False,
-        position: Optional[List[float]] = None,
-        set_mesh_direction: Optional[List[float]] = None,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        verbose: Optional[bool] = False,
-        **kwargs,
+        wall_keep_prob: float = 0.5,
+        wall_material: Optional[Material] = None,
+        **kwargs: Any,
     ):
-
-        if seed is None:
-            seed = np.random.randint(0, 100000)
-            if verbose:
-                logger.info("Seed:", seed)
-
-        # Seeding
-        np.random.seed(seed)
-
-        if sample_map is not None and not isinstance(sample_map, np.ndarray):
-            sample_map = np.array(sample_map)
-
-        if specific_map is not None and not isinstance(specific_map, np.ndarray):
-            specific_map = np.array(specific_map)
-
-        if algorithm_args is None:
-            algorithm_args = {}
-
-        # Handle when user doesn't pass arguments properly
-        if (tiles is None or neighbors is None) and sample_map is None and specific_map is None:
-            raise ValueError("Insert tiles / neighbors or a map to sample from.")
-
-        # Get coordinates and image from procedural generation
-        all_args = {
-            "width": width,
-            "height": height,
-            "sample_map": sample_map,
-            "tiles": tiles,
-            "neighbors": neighbors,
-            "weights": weights,
-            "symmetries": symmetries,
-            **algorithm_args,
-        }
-
-        if shallow:
-            if specific_map is None:
-                map_2ds = generate_2d_map(**all_args)
-                # We take the first map (if nb_samples > 1), since this object has
-                # support for a single map for now
-                self.map_2d = map_2ds[0]
-
-            else:
-                self.map_2d = specific_map
-
-        else:
-            # Saves these for other functions that might use them
-            # We take index 0 since generate_map is now vectorized, but we don't have
-            # support for multiple maps on this object yet.
-            coordinates, map_2ds = generate_map(specific_map=specific_map, **all_args)
-            self.coordinates, self.map_2d = coordinates[0], map_2ds[0]
-
-            # If it is a structured grid, extract the surface mesh (PolyData)
-            mesh = pv.StructuredGrid(*self.coordinates).extract_surface()
-            original_mesh_direction = (0, 1, 0)
-            pv.translate(mesh, (0, 0, 0), new_direction=original_mesh_direction)
-
-            super().__init__(
-                mesh=mesh,
-                name=name,
-                position=position,
-                is_actor=is_actor,
-                set_mesh_direction=set_mesh_direction,
-                original_mesh_direction=original_mesh_direction,
-                with_rigid_body=with_rigid_body,
-                with_articulation_body=with_articulation_body,
-                parent=parent,
-                children=children,
-                **kwargs,
-            )
-
-    def generate_3D(
-        self,
-        name: Optional[str] = None,
-        is_actor: Optional[bool] = False,
-        with_rigid_body: Optional[bool] = False,
-        with_articulation_body: Optional[bool] = False,
-        parent: Optional[Asset] = None,
-        children: Optional[List[Asset]] = None,
-        **kwargs,
-    ):
-        """
-        Function for creating the mesh in case the creation of map was shallow.
-        """
-        coordinates, _ = generate_map(specific_map=self.map_2d)
-        self.coordinates = coordinates[0]
-
-        # If it is a structured grid, extract the surface mesh (PolyData)
-        mesh = pv.StructuredGrid(*self.coordinates).extract_surface()
-        super().__init__(
-            mesh=mesh,
-            name=name,
-            is_actor=is_actor,
-            with_rigid_body=with_rigid_body,
-            with_articulation_body=with_articulation_body,
-            parent=parent,
-            children=children,
-            **kwargs,
-        )
-
-
-class ProcGenPrimsMaze3D(Asset):
-    __NEW_ID = itertools.count()  # Singleton to count instances of the classes for automatic naming
-
-    def __init__(self, width: int, depth: int, name=None, wall_keep_prob=0.5, wall_material=None, **kwargs):
         self.width = width
         self.depth = depth
         self.wall_keep_prob = wall_keep_prob * 10
         if wall_material is None:
-            wall_material = Material(base_color=(0.8, 0.8, 0.8))
+            wall_material = Material(base_color=[0.8, 0.8, 0.8])
         self.wall_material = wall_material
         if name is not None:
             self.name = name
